@@ -319,7 +319,6 @@ mx_combo_box_get_preferred_height (ClutterActor *actor,
                                    gfloat       *natural_height_p)
 {
   MxComboBoxPrivate *priv = MX_COMBO_BOX (actor)->priv;
-  gfloat width;
   gfloat min_label_h, nat_label_h;
   gfloat min_icon_h = 0, nat_icon_h = 0;
   gfloat min_marker_h = 0, nat_marker_h = 0;
@@ -328,8 +327,6 @@ mx_combo_box_get_preferred_height (ClutterActor *actor,
 
   mx_widget_get_padding (MX_WIDGET (actor), &padding);
 
-  width = for_width - padding.left - padding.right;
-
   clutter_actor_get_preferred_height (priv->label,
                                       -1,
                                       &min_label_h,
@@ -562,6 +559,15 @@ mx_combo_box_key_press_event (ClutterActor    *actor,
     }
 }
 
+static void
+mx_combo_box_apply_style (MxWidget *widget,
+                          MxStyle  *style)
+{
+  MxComboBoxPrivate *priv = MX_COMBO_BOX (widget)->priv;
+
+  if (priv->icon != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->icon), style);
+}
 
 static void
 mx_combo_box_style_changed (MxComboBox *combo, MxStyleChangedFlags flags)
@@ -612,6 +618,7 @@ mx_combo_box_class_init (MxComboBoxClass *klass)
   GParamSpec *pspec;
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  MxWidgetClass *widget_class = MX_WIDGET_CLASS (klass);
 
   g_type_class_add_private (klass, sizeof (MxComboBoxPrivate));
 
@@ -631,6 +638,8 @@ mx_combo_box_class_init (MxComboBoxClass *klass)
   actor_class->button_press_event = mx_combo_box_button_press_event;
   actor_class->key_press_event = mx_combo_box_key_press_event;
 
+  widget_class->apply_style = mx_combo_box_apply_style;
+
   pspec = g_param_spec_string ("active-text",
                                "Active Text",
                                "Text currently displayed in the combo box"
@@ -1026,9 +1035,9 @@ mx_combo_box_set_index (MxComboBox *box,
  *
  * Get the index of the last item selected
  *
- * Returns: const gint
+ * Returns: gint
  */
-const gint
+gint
 mx_combo_box_get_index (MxComboBox *box)
 {
   g_return_val_if_fail (MX_IS_COMBO_BOX (box), 0);
