@@ -27,6 +27,7 @@
 #include "mx-marshal.h"
 #include "mx-texture-cache.h"
 #include "mx-private.h"
+#include "mx-settings.h"
 
 G_DEFINE_TYPE (MxIconTheme, mx_icon_theme, G_TYPE_OBJECT)
 
@@ -59,6 +60,8 @@ typedef struct
 
 struct _MxIconThemePrivate
 {
+  guint       override_theme : 1;
+
   GList      *search_paths;
   GHashTable *icon_hash;
   GHashTable *theme_path_hash;
@@ -319,6 +322,23 @@ mx_icon_theme_load_fallbacks (MxIconTheme *theme,
     }
 }
 
+static void
+mx_icon_theme_changed_cb (MxSettings  *settings,
+                          GParamSpec  *pspec,
+                          MxIconTheme *self)
+{
+  gchar *theme;
+
+  if (self->priv->override_theme)
+    return;
+
+  g_object_get (settings, "icon-theme", &theme, NULL);
+  mx_icon_theme_set_theme_name (self, theme);
+  g_free (theme);
+
+  self->priv->override_theme = FALSE;
+}
+
 static void
 mx_icon_theme_init (MxIconTheme *self)
 {
@@ -368,10 +388,16 @@ mx_icon_theme_init (MxIconTheme *self)
     g_warning ("Error loading fallback icon theme");
 
   theme = g_getenv ("MX_ICON_THEME");
-  if (!theme)
-    theme = "moblin";
+  if (theme)
+    mx_icon_theme_set_theme_name (self, theme);
+  else
+    {
+      MxSettings *settings = mx_settings_get_default ();
+      g_signal_connect (settings, "notify::icon-theme",
+                        G_CALLBACK (mx_icon_theme_changed_cb), self);
+      mx_icon_theme_changed_cb (settings, NULL, self);
+    }
 
-  mx_icon_theme_set_theme_name (self, theme);
 }
 
 /**
@@ -425,13 +451,16 @@ mx_icon_theme_get_theme_name (MxIconTheme *theme)
 /**
  * mx_icon_theme_set_theme_name:
  * @theme: A #MxIconTheme
- * @theme_name: the name of an icon theme to load
+ * @theme_name: the name of an icon theme to load, or %NULL
  *
  * Set the value of the #MxIconTheme:theme-name property. This will cause the
  * icon theme to be loaded if it differs from the existing theme name. If the
  * theme could not be loaded, it will fall back to using the default icon theme
  * (hicolor).
  *
+ * This will override the system's theme setting. To revert to the system
+ * icon theme, this function can be called with a %NULL @theme_name argument.
+ *
  */
 void
 mx_icon_theme_set_theme_name (MxIconTheme *theme,
@@ -440,10 +469,28 @@ mx_icon_theme_set_theme_name (MxIconTheme *theme,
   MxIconThemePrivate *priv;
 
   g_return_if_fail (MX_IS_ICON_THEME (theme));
-  g_return_if_fail (theme_name != NULL);
 
   priv = theme->priv;
 
+  if (!theme_name)
+    {
+      if (priv->override_theme)
+        {
+          gchar *system_theme = NULL;
+          MxSettings *settings = mx_settings_get_default ();
+
+          g_object_get (settings, "icon-theme", &system_theme, NULL);
+          priv->override_theme = FALSE;
+          mx_icon_theme_set_theme_name (theme, system_theme);
+          priv->override_theme = FALSE;
+          g_free (system_theme);
+        }
+
+      return;
+    }
+
+  priv->override_theme = TRUE;
+
   if (g_str_equal (theme_name, "hicolor"))
     return;
 
@@ -475,7 +522,7 @@ mx_icon_theme_set_theme_name (MxIconTheme *theme,
 
   if (!priv->theme_file)
     {
-      g_warning ("Error loading theme");
+      g_warning ("Error loading \"%s\" icon theme", priv->theme);
       return;
     }
 
@@ -877,7 +924,7 @@ mx_icon_theme_lookup_internal (MxIconTheme *theme,
  *
  * If the icon is available, returns a #CoglHandle of the icon.
  *
- * Return value: a #CoglHandle of the icon, or %NULL.
+ * Return value: (transfer none): a #CoglHandle of the icon, or %NULL.
  */
 CoglHandle
 mx_icon_theme_lookup (MxIconTheme *theme,
@@ -948,6 +995,15 @@ mx_icon_theme_get_search_paths (MxIconTheme *theme)
   return theme->priv->search_paths;
 }
 
+/**
+ * mx_icon_theme_set_search_paths:
+ * @theme: a #MxIconTheme
+ * @paths: (element-type utf8): a list of search paths
+ *
+ * Sets the directories the #MxIconTheme will search in to find icons.
+ * By default, it will look in the default system and local icon
+ * directories.
+ */
 void
 mx_icon_theme_set_search_paths (MxIconTheme *theme,
                                 const GList *paths)
