@@ -258,6 +258,25 @@ mx_scroll_bar_unmap (ClutterActor *actor)
     clutter_actor_unmap (priv->handle);
 }
 
+static void
+mx_scroll_bar_apply_style (MxWidget *widget,
+                           MxStyle  *style)
+{
+  MxScrollBarPrivate *priv = MX_SCROLL_BAR (widget)->priv;
+
+  if (priv->bw_stepper != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->bw_stepper), style);
+
+  if (priv->fw_stepper != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->fw_stepper), style);
+
+  if (priv->trough != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->trough), style);
+
+  if (priv->handle != NULL)
+    mx_stylable_set_style (MX_STYLABLE (priv->handle), style);
+}
+
 static void
 mx_scroll_bar_get_preferred_width (ClutterActor *actor,
                                    gfloat        for_height,
@@ -537,13 +556,11 @@ mx_scroll_bar_constructor (GType                  type,
   GObjectClass *gobject_class;
   GObject *obj;
   MxScrollBar *bar;
-  MxScrollBarPrivate *priv;
 
   gobject_class = G_OBJECT_CLASS (mx_scroll_bar_parent_class);
   obj = gobject_class->constructor (type, n_properties, properties);
 
   bar  = MX_SCROLL_BAR (obj);
-  priv = MX_SCROLL_BAR_GET_PRIVATE (bar);
 
   g_signal_connect (bar, "notify::reactive",
                     G_CALLBACK (bar_reactive_notify_cb), NULL);
@@ -600,6 +617,7 @@ mx_scroll_bar_class_init (MxScrollBarClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
+  MxWidgetClass *widget_class = MX_WIDGET_CLASS (klass);
   GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (MxScrollBarPrivate));
@@ -618,6 +636,8 @@ mx_scroll_bar_class_init (MxScrollBarClass *klass)
   actor_class->map            = mx_scroll_bar_map;
   actor_class->unmap          = mx_scroll_bar_unmap;
 
+  widget_class->apply_style = mx_scroll_bar_apply_style;
+
   g_object_class_install_property
                  (object_class,
                  PROP_ADJUSTMENT,
