@@ -45,6 +45,7 @@
 #include "mx-marshal.h"
 #include "mx-private.h"
 #include "mx-stylable.h"
+#include "mx-settings.h"
 
 enum
 {
@@ -141,7 +142,8 @@ mx_stylable_base_init (gpointer g_iface)
 
   pspec = g_param_spec_string ("style-pseudo-class",
                                "Style Pseudo Class",
-                               "Pseudo class, such as current state",
+                               "List of pseudo class, such as current state,"
+                               "separated by ':'.",
                                "",
                                MX_PARAM_READWRITE);
   g_object_interface_install_property (g_iface, pspec);
@@ -155,7 +157,7 @@ mx_stylable_base_init (gpointer g_iface)
    * properties have changed.
    */
   stylable_signals[STYLE_CHANGED] =
-    g_signal_new (I_("style-changed"),
+    g_signal_new (g_intern_static_string ("style-changed"),
                   iface_type,
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (MxStylableIface, style_changed),
@@ -165,7 +167,7 @@ mx_stylable_base_init (gpointer g_iface)
 
 #if 0
   stylable_signals[STYLE_NOTIFY] =
-    g_signal_new (I_("style-notify"),
+    g_signal_new (g_intern_static_string ("style-notify"),
                   iface_type,
                   G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED
                   | G_SIGNAL_NO_HOOKS | G_SIGNAL_ACTION,
@@ -191,13 +193,87 @@ mx_stylable_get_type (void)
       };
 
       our_type = g_type_register_static (G_TYPE_INTERFACE,
-                                         I_("MxStylable"),
+                                         g_intern_static_string ("MxStylable"),
                                          &stylable_info, 0);
     }
 
   return our_type;
 }
 
+static void
+_mx_stylable_prepend_style_string (GString    *string,
+                                   MxStylable *stylable)
+{
+  GType type_id;
+  gchar *style_string;
+  const gchar *type, *id, *class, *pseudo_class;
+
+  type_id = G_OBJECT_CLASS_TYPE (G_OBJECT_GET_CLASS (stylable));
+  type = g_type_name (type_id);
+
+  id = clutter_actor_get_name (CLUTTER_ACTOR (stylable));
+
+  class = mx_stylable_get_style_class (stylable);
+
+  pseudo_class = mx_stylable_get_style_pseudo_class (stylable);
+
+  style_string = g_strdup_printf ("%s#%s.%s:%s>",
+                                  type,
+                                  id ? id : "",
+                                  class ? class : "",
+                                  pseudo_class ? pseudo_class : "");
+  g_string_prepend (string, style_string);
+  g_free (style_string);
+}
+
+gchar *
+_mx_stylable_get_style_string (MxStylable *stylable)
+{
+  GType type_id;
+  gchar *cstring;
+  GString *string;
+  const gchar *type, *id, *class, *pseudo_class;
+
+  /* Create a string that contains all the properties of a
+   * Stylable that can be matched against in the CSS.
+   */
+  type_id = G_OBJECT_CLASS_TYPE (G_OBJECT_GET_CLASS (stylable));
+  type = g_type_name (type_id);
+
+  id = clutter_actor_get_name (CLUTTER_ACTOR (stylable));
+
+  class = mx_stylable_get_style_class (stylable);
+
+  pseudo_class = mx_stylable_get_style_pseudo_class (stylable);
+
+  string = g_string_new (type);
+
+  /* Add the actor hierarchy */
+  if (CLUTTER_IS_ACTOR (stylable))
+    {
+      ClutterActor *parent =
+        clutter_actor_get_parent (CLUTTER_ACTOR (stylable));
+
+      while (parent)
+        {
+          if (MX_IS_STYLABLE (parent))
+            _mx_stylable_prepend_style_string (string, MX_STYLABLE (parent));
+
+          parent = clutter_actor_get_parent (parent);
+        }
+    }
+
+  g_string_append_printf (string, "#%s.%s:%s",
+                          id ? id : "",
+                          class ? class : "",
+                          pseudo_class ? pseudo_class : "");
+
+  cstring = string->str;
+  g_string_free (string, FALSE);
+
+  return cstring;
+}
+
 #if 0
 void
 mx_stylable_freeze_notify (MxStylable *stylable)
@@ -512,6 +588,63 @@ mx_stylable_get (MxStylable  *stylable,
   va_end (args);
 }
 
+static gboolean
+_set_from_xsettings (GParamSpec *pspec,
+                     GValue     *value)
+{
+  gchar *font_string;
+  gint font_size;
+  PangoFontDescription *descr;
+  MxSettings *settings = mx_settings_get_default ();
+  gboolean result = FALSE;
+
+  if (!strcmp (pspec->name, "font-family")
+      || !strcmp (pspec->name, "font-size"))
+    {
+      const gchar *fn;
+      PangoFontMask set_fields;
+
+      g_object_get (settings, "font-name", &font_string, NULL);
+
+      if (!font_string)
+        return FALSE;
+
+      descr = pango_font_description_from_string (font_string);
+
+      set_fields = pango_font_description_get_set_fields (descr);
+
+      /* font name */
+      if ((set_fields & PANGO_FONT_MASK_FAMILY)
+          && !strcmp (pspec->name, "font-family")
+          && G_VALUE_HOLDS_STRING (value))
+        {
+          fn = pango_font_description_get_family (descr);
+          g_value_set_string (value, fn);
+          result = TRUE;
+        }
+      /* font size */
+      else if ((set_fields & PANGO_FONT_MASK_SIZE)
+               && !strcmp (pspec->name, "font-size")
+               && G_VALUE_HOLDS_INT (value))
+        {
+          font_size = pango_font_description_get_size (descr) / PANGO_SCALE;
+          if (!pango_font_description_get_size_is_absolute (descr))
+            {
+              ClutterBackend *backend = clutter_get_default_backend ();
+              gdouble res = clutter_backend_get_resolution (backend);
+              font_size = font_size * res / 72.0;
+            }
+          g_value_set_int (value, font_size);
+          result = TRUE;
+        }
+
+      pango_font_description_free (descr);
+      descr = NULL;
+    }
+
+  return result;
+}
+
 /**
  * mx_stylable_get_default_value:
  * @stylable: a #MxStylable
@@ -550,7 +683,11 @@ mx_stylable_get_default_value (MxStylable  *stylable,
     }
 
   g_value_init (value_out, G_PARAM_SPEC_VALUE_TYPE (pspec));
-  g_param_value_set_default (pspec, value_out);
+
+  /* default font values come from xsettings if possible */
+  if (!_set_from_xsettings (pspec, value_out))
+    g_param_value_set_default (pspec, value_out);
+
   return TRUE;
 }
 
@@ -632,7 +769,7 @@ mx_stylable_set_style (MxStylable *stylable,
                            data,
                            (GDestroyNotify) disconnect_style_changed_signal);
 
-  mx_stylable_style_changed (stylable, MX_STYLE_CHANGED_NONE);
+  mx_stylable_style_changed (stylable, MX_STYLE_CHANGED_INVALIDATE_CACHE);
 
   g_object_notify (G_OBJECT (stylable), "style");
 }
@@ -641,7 +778,8 @@ mx_stylable_set_style (MxStylable *stylable,
  * mx_stylable_get_style_pseudo_class:
  * @stylable: a #MxStylable
  *
- * Get the current style pseudo class
+ * Get the current style pseudo class. This can contain multiple pseudo class
+ * names, separated by ':'.
  *
  * Returns: the pseudo class string. The string is owned by the #MxWidget and
  * should not be modified or freed.
@@ -669,7 +807,8 @@ mx_stylable_get_style_pseudo_class (MxStylable *stylable)
  * @stylable: a #MxStylable
  * @pseudo_class: a new pseudo class string
  *
- * Set the style pseudo class
+ * Set the style pseudo class. The string can contain multiple pseudo class
+ * names, separated by ':'.
  */
 void
 mx_stylable_set_style_pseudo_class (MxStylable  *stylable,
@@ -689,6 +828,132 @@ mx_stylable_set_style_pseudo_class (MxStylable  *stylable,
                g_type_name (G_OBJECT_TYPE (stylable)));
 }
 
+/**
+ * mx_stylable_style_pseudo_class_contains:
+ * @stylable: A #MxStylable
+ * @pseudo_class: A pseudo-class name
+ *
+ * Check if the given pseudo-class name is contained in the list of
+ * set pseudo classes on this #MxStylable object.
+ *
+ * Returns: %TRUE if the given pseudo-class is set, %FALSE otherwise
+ *
+ * Since: 1.2
+ */
+gboolean
+mx_stylable_style_pseudo_class_contains (MxStylable  *stylable,
+                                         const gchar *pseudo_class)
+{
+  const gchar *old_class, *match;
+
+  g_return_val_if_fail (MX_IS_STYLABLE (stylable), FALSE);
+  g_return_val_if_fail (pseudo_class != NULL, FALSE);
+
+  old_class = mx_stylable_get_style_pseudo_class (stylable);
+
+  if (old_class && pseudo_class && (match = strstr (old_class, pseudo_class)))
+    {
+      if ((match == old_class) ||
+           (match[-1] == ':'))
+        {
+          size_t length = strlen (match);
+          if ((match[length] == ':') ||
+              (match[length] == '\0'))
+            return TRUE;
+        }
+    }
+
+  return FALSE;
+}
+
+/**
+ * mx_stylable_style_pseudo_class_add:
+ * @stylable: A #MxStylable
+ * @new_class: A pseudo-class name to add
+ *
+ * Add a pseudo-class name to the list of pseudo classes, contained in the
+ * #MxStylable:style-pseudo-class property.
+ *
+ * Since: 1.2
+ */
+void
+mx_stylable_style_pseudo_class_add (MxStylable  *stylable,
+                                    const gchar *new_class)
+{
+  const gchar *old_class;
+  gchar *tmp;
+
+  g_return_if_fail (MX_IS_STYLABLE (stylable));
+  g_return_if_fail (new_class != NULL);
+
+  /* check if the pseudo class already contains new_class */
+  if (mx_stylable_style_pseudo_class_contains (stylable, new_class))
+    return;
+
+  old_class = mx_stylable_get_style_pseudo_class (stylable);
+
+  /* add the new pseudo class */
+  if (old_class)
+    tmp = g_strconcat (old_class, ":", new_class, NULL);
+  else
+    tmp = g_strdup (new_class);
+
+  mx_stylable_set_style_pseudo_class (stylable, tmp);
+
+  g_free (tmp);
+}
+
+/**
+ * mx_stylable_style_pseudo_class_remove:
+ * @stylable: An #MxStylable
+ * @remove_class: A pseudo class name to remove
+ *
+ * Remove the specified pseudo class name from the list of pseudo classes
+ * contained in the #MxStylable:style-pseudo-class property.
+ *
+ * Since: 1.2
+ */
+void
+mx_stylable_style_pseudo_class_remove (MxStylable  *stylable,
+                                       const gchar *remove_class)
+{
+  const gchar *old_class;
+  gchar *tmp;
+  gchar **list;
+  gint i, len;
+
+  g_return_if_fail (MX_IS_STYLABLE (stylable));
+  g_return_if_fail (remove_class != NULL);
+
+  /* check if the pseudo class does not container remove_class */
+  if (!mx_stylable_style_pseudo_class_contains (stylable, remove_class))
+    return;
+
+  old_class = mx_stylable_get_style_pseudo_class (stylable);
+
+  /* remove the old pseudo class */
+  list = g_strsplit (old_class, ":", -1);
+
+  len = g_strv_length (list);
+  tmp = NULL;
+  for (i = 0; i < len; i++)
+    {
+      /* skip over any instances of remove_class */
+      if (!strcmp (list[i], remove_class))
+        continue;
+
+      if (tmp)
+        tmp = g_strconcat (list[i], ":", tmp, NULL);
+      else
+        tmp = g_strdup (list[i]);
+    }
+
+  mx_stylable_set_style_pseudo_class (stylable, tmp);
+
+  g_strfreev (list);
+  g_free (tmp);
+}
+
 /**
  * mx_stylable_get_style_class:
  * @stylable: a #MxStylable
@@ -743,7 +1008,7 @@ mx_stylable_set_style_class (MxStylable  *stylable,
 static void
 mx_stylable_property_changed_notify (MxStylable *stylable)
 {
-  mx_stylable_style_changed (stylable, MX_STYLE_CHANGED_NONE);
+  mx_stylable_style_changed (stylable, MX_STYLE_CHANGED_INVALIDATE_CACHE);
 }
 
 static void
@@ -755,7 +1020,8 @@ mx_stylable_parent_set_notify (ClutterActor *actor,
   /* check the actor has a new parent */
   if (new_parent)
     {
-      mx_stylable_style_changed (MX_STYLABLE (actor), MX_STYLE_CHANGED_NONE);
+      mx_stylable_style_changed (MX_STYLABLE (actor),
+                                 MX_STYLE_CHANGED_INVALIDATE_CACHE);
     }
 }
 
@@ -777,11 +1043,21 @@ mx_stylable_style_changed_internal (MxStylable          *stylable,
 {
 
   /* don't update stylables until they are mapped (unless ensure is set) */
-  if (!CLUTTER_ACTOR_IS_MAPPED (CLUTTER_ACTOR (stylable)) &&
+  if (G_LIKELY (CLUTTER_IS_ACTOR (stylable)) &&
+      !CLUTTER_ACTOR_IS_MAPPED (CLUTTER_ACTOR (stylable)) &&
       !(flags & MX_STYLE_CHANGED_FORCE))
     return;
 
-  _mx_style_invalidate_cache (stylable);
+  if (flags & MX_STYLE_CHANGED_INVALIDATE_CACHE)
+    _mx_style_invalidate_cache (stylable);
+
+  /* If the parent style has changed, child cache needs to be
+   * invalidated. This needs to happen for internal children as
+   * well, which is why it's here and not in the container block
+   * lower down.
+   */
+  flags |= MX_STYLE_CHANGED_INVALIDATE_CACHE;
+
   g_signal_emit (stylable, stylable_signals[STYLE_CHANGED], 0, flags);
 
   /* propagate the style-changed signal to children, since their style may
