@@ -95,6 +95,8 @@ struct _MxTablePrivate
 {
   GList *children;
 
+  guint   ignore_css_col_spacing : 1;
+  guint   ignore_css_row_spacing : 1;
   gint    col_spacing;
   gint    row_spacing;
 
@@ -115,13 +117,17 @@ struct _MxTablePrivate
 
 static void mx_container_iface_init (ClutterContainerIface *iface);
 static void mx_focusable_iface_init (MxFocusableIface *iface);
+static void mx_stylable_iface_init (MxStylableIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (MxTable, mx_table, MX_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
                                                 mx_container_iface_init)
+                         G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
+                                                mx_stylable_iface_init)
                          G_IMPLEMENT_INTERFACE (MX_TYPE_FOCUSABLE,
                                                 mx_focusable_iface_init));
 
+
 static ClutterActor*
 mx_table_find_actor_at (MxTable *table,
                         int      row,
@@ -168,6 +174,8 @@ mx_table_move_focus (MxFocusable      *focusable,
   if (!childlink)
     return NULL;
 
+  priv->last_focus = from;
+
   child_actor = CLUTTER_ACTOR (childlink->data);
   child_meta = (MxTableChild *) clutter_container_get_child_meta (CLUTTER_CONTAINER (focusable),
                                                                   child_actor);
@@ -185,10 +193,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                                                    MX_FOCUS_HINT_FIRST);
 
               if (focused)
-                {
-                  priv->last_focus = (MxFocusable *)l->data;
-                  return focused;
-                }
+                return focused;
             }
         }
 
@@ -205,10 +210,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                                                    MX_FOCUS_HINT_LAST);
 
               if (focused)
-                {
-                  priv->last_focus = (MxFocusable *)l->data;
-                  return focused;
-                }
+                return focused;
             }
         }
 
@@ -234,10 +236,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                   focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
                                                        MX_FOCUS_HINT_FIRST);
                   if (focused)
-                    {
-                      priv->last_focus = (MxFocusable *)found;
-                      break;
-                    }
+                    break;
                 }
 
               child_meta = (MxTableChild *) clutter_container_get_child_meta
@@ -272,10 +271,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                   focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
                                                        MX_FOCUS_HINT_FIRST);
                   if (focused)
-                    {
-                      priv->last_focus = (MxFocusable *)found;
-                      break;
-                    }
+                    break;
                 }
 
               child_meta = (MxTableChild *) clutter_container_get_child_meta
@@ -309,10 +305,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                   focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
                                                        MX_FOCUS_HINT_FIRST);
                   if (focused)
-                    {
-                      priv->last_focus = (MxFocusable *)found;
-                      break;
-                    }
+                    break;
                 }
 
               child_meta = (MxTableChild *) clutter_container_get_child_meta
@@ -347,10 +340,7 @@ mx_table_move_focus (MxFocusable      *focusable,
                   focused = mx_focusable_accept_focus (MX_FOCUSABLE (found),
                                                        MX_FOCUS_HINT_FIRST);
                   if (focused)
-                    {
-                      priv->last_focus = (MxFocusable *)found;
-                      break;
-                    }
+                    break;
                 }
 
               child_meta = (MxTableChild *) clutter_container_get_child_meta
@@ -436,9 +426,11 @@ mx_container_add_actor (ClutterContainer *container,
 
   clutter_actor_set_parent (actor, CLUTTER_ACTOR (container));
 
-
   priv->children = g_list_append (priv->children, actor);
 
+  /* default position of the actor is 0, 0 */
+  _mx_table_update_row_col (MX_TABLE (container), 0, 0);
+
   clutter_actor_queue_relayout (CLUTTER_ACTOR (container));
 
   g_signal_emit_by_name (container, "actor-added", actor);
@@ -509,8 +501,36 @@ mx_container_lower (ClutterContainer *container,
                     ClutterActor     *actor,
                     ClutterActor     *sibling)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  gint i;
+  GList *c, *position, *actor_link = NULL;
+
+  MxTablePrivate *priv = MX_TABLE (container)->priv;
+
+  if (priv->children && (priv->children->data == actor))
+    return;
+
+  position = priv->children;
+  for (c = priv->children, i = 0; c; c = c->next, i++)
+    {
+      if (c->data == actor)
+        actor_link = c;
+      if (c->data == sibling)
+        position = c;
+    }
+
+  if (!actor_link)
+    {
+      g_warning (G_STRLOC ": Actor of type '%s' is not a child of container "
+                 "of type '%s'",
+                 g_type_name (G_OBJECT_TYPE (actor)),
+                 g_type_name (G_OBJECT_TYPE (container)));
+      return;
+    }
+
+  priv->children = g_list_delete_link (priv->children, actor_link);
+  priv->children = g_list_insert_before (priv->children, position, actor);
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -518,15 +538,61 @@ mx_container_raise (ClutterContainer *container,
                     ClutterActor     *actor,
                     ClutterActor     *sibling)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  gint i;
+  GList *c, *actor_link = NULL;
+
+  gint position = -1;
+  MxTablePrivate *priv = MX_TABLE (container)->priv;
+
+  for (c = priv->children, i = 0; c; c = c->next, i++)
+    {
+      if (c->data == actor)
+        actor_link = c;
+      if (c->data == sibling)
+        position = i;
+    }
+
+  if (!actor_link)
+    {
+      g_warning (G_STRLOC ": Actor of type '%s' is not a child of container "
+                 "of type '%s'",
+                 g_type_name (G_OBJECT_TYPE (actor)),
+                 g_type_name (G_OBJECT_TYPE (container)));
+      return;
+    }
+
+  if (!actor_link->next)
+    return;
+
+  priv->children = g_list_delete_link (priv->children, actor_link);
+  priv->children = g_list_insert (priv->children, actor, position);
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
+}
+
+static gint
+mx_table_depth_sort_cb (gconstpointer a,
+                        gconstpointer b)
+{
+  gfloat depth_a = clutter_actor_get_depth ((ClutterActor *)a);
+  gfloat depth_b = clutter_actor_get_depth ((ClutterActor *)a);
+
+  if (depth_a < depth_b)
+    return -1;
+  else if (depth_a > depth_b)
+    return 1;
+  else
+    return 0;
 }
 
 static void
 mx_container_sort_depth_order (ClutterContainer *container)
 {
-  /* XXX: not yet implemented */
-  g_warning ("%s() not yet implemented", __FUNCTION__);
+  MxTablePrivate *priv = MX_TABLE (container)->priv;
+
+  priv->children = g_list_sort (priv->children, mx_table_depth_sort_cb);
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -541,6 +607,34 @@ mx_container_iface_init (ClutterContainerIface *iface)
   iface->child_meta_type = MX_TYPE_TABLE_CHILD;
 }
 
+/* MxStylable implementation */
+static void
+mx_stylable_iface_init (MxStylableIface *iface)
+{
+  static gboolean is_initialized = FALSE;
+
+  if (G_UNLIKELY (is_initialized == FALSE))
+    {
+      GParamSpec *pspec;
+
+      is_initialized = TRUE;
+
+      pspec = g_param_spec_uint ("x-mx-column-spacing",
+                                 "Column Spacing",
+                                 "The size of the column spacing",
+                                 0, G_MAXUINT, 0,
+                                 MX_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_TABLE, pspec);
+
+      pspec = g_param_spec_uint ("x-mx-row-spacing",
+                                 "Row Spacing",
+                                 "The size of the row spacing",
+                                 0, G_MAXUINT, 0,
+                                 MX_PARAM_READWRITE);
+      mx_stylable_iface_install_property (iface, MX_TYPE_TABLE, pspec);
+    }
+}
+
 /* MxTable Class Implementation */
 
 static void
@@ -683,7 +777,7 @@ mx_table_calculate_col_widths (MxTable *table,
       clutter_actor_get_preferred_width (child, -1, &c_min, &c_pref);
 
       col->min_size = MAX (col->min_size, c_min);
-      col->pref_size = MAX (col->pref_size, c_pref);
+      col->final_size = col->pref_size = MAX (col->pref_size, c_pref);
       col->expand = MAX (col->expand, meta->x_expand);
     }
 
@@ -972,7 +1066,7 @@ mx_table_calculate_row_heights (MxTable *table,
                                           &c_min, &c_pref);
 
       row->min_size = MAX (row->min_size, c_min);
-      row->pref_size = MAX (row->pref_size, c_pref);
+      row->final_size = row->pref_size = MAX (row->pref_size, c_pref);
       row->expand = MAX (row->expand, meta->y_expand);
     }
 
@@ -983,7 +1077,6 @@ mx_table_calculate_row_heights (MxTable *table,
     {
       MxTableChild *meta;
       ClutterActor *child;
-      DimensionData *row;
       gfloat c_min, c_pref;
       gfloat min_height, pref_height;
       gint start_row, end_row;
@@ -1000,7 +1093,6 @@ mx_table_calculate_row_heights (MxTable *table,
       if (meta->row_span < 2)
         continue;
 
-      row = &rows[meta->row];
       start_row = meta->row;
       end_row = meta->row + meta->row_span - 1;
 
@@ -1230,7 +1322,7 @@ mx_table_preferred_allocate (ClutterActor          *self,
 {
   GList *list;
   gint row_spacing, col_spacing;
-  gint i, table_width, table_height;
+  gint i;
   MxTable *table;
   MxTablePrivate *priv;
   MxPadding padding;
@@ -1244,14 +1336,6 @@ mx_table_preferred_allocate (ClutterActor          *self,
   col_spacing = (priv->col_spacing);
   row_spacing = (priv->row_spacing);
 
-
-  table_height = (int)(box->y2 - box->y1
-                       - padding.top
-                       - padding.bottom);
-  table_width = (int)(box->x2 - box->x1
-                      - padding.right
-                      - padding.left);
-
   mx_table_calculate_dimensions (table, box->x2 - box->x1, box->y2 - box->y1);
 
   rows = (DimensionData *) priv->rows->data;
@@ -1628,6 +1712,26 @@ mx_table_class_init (MxTableClass *klass)
                                    pspec);
 }
 
+static void
+mx_table_style_changed (MxWidget *widget,
+                        gpointer  userdata)
+{
+  MxTable *table = MX_TABLE (widget);
+  MxTablePrivate *priv = table->priv;
+  guint row_spacing, col_spacing;
+
+  mx_stylable_get (MX_STYLABLE (widget),
+                   "x-mx-column-spacing", &col_spacing,
+                   "x-mx-row-spacing", &row_spacing,
+                   NULL);
+
+  if (!priv->ignore_css_col_spacing)
+    priv->col_spacing = col_spacing;
+
+  if (!priv->ignore_css_row_spacing)
+    priv->row_spacing = row_spacing;
+}
+
 static void
 mx_table_init (MxTable *table)
 {
@@ -1638,6 +1742,9 @@ mx_table_init (MxTable *table)
 
   table->priv->columns = g_array_new (FALSE, TRUE, sizeof (DimensionData));
   table->priv->rows = g_array_new (FALSE, TRUE, sizeof (DimensionData));
+
+  g_signal_connect (table, "style-changed",
+                    G_CALLBACK (mx_table_style_changed), NULL);
 }
 
 /* used by MxTableChild to update row/column count */
@@ -1690,6 +1797,8 @@ mx_table_set_column_spacing (MxTable *table,
     {
       priv->col_spacing = spacing;
 
+      priv->ignore_css_col_spacing = TRUE;
+
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
 
       g_object_notify (G_OBJECT (table), "column-spacing");
@@ -1718,6 +1827,8 @@ mx_table_set_row_spacing (MxTable *table,
     {
       priv->row_spacing = spacing;
 
+      priv->ignore_css_row_spacing = TRUE;
+
       clutter_actor_queue_relayout (CLUTTER_ACTOR (table));
 
       g_object_notify (G_OBJECT (table), "row-spacing");
