@@ -656,16 +656,16 @@ mx_viewport_get_origin (MxViewport *viewport,
 
 void
 mx_viewport_set_sync_adjustments (MxViewport *viewport,
-                                  gboolean    sync)
+                                  gboolean    sync_adjustments)
 {
   MxViewportPrivate *priv;
 
   g_return_if_fail (MX_IS_VIEWPORT (viewport));
 
   priv = viewport->priv;
-  if (priv->sync_adjustments != sync)
+  if (priv->sync_adjustments != sync_adjustments)
     {
-      priv->sync_adjustments = sync;
+      priv->sync_adjustments = sync_adjustments;
       g_object_notify (G_OBJECT (viewport), "sync-adjustments");
     }
 }
