@@ -30,6 +30,7 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <libintl.h>
 
 #include <clutter/clutter.h>
 
@@ -42,8 +43,7 @@
 #include "mx-texture-frame.h"
 #include "mx-tooltip.h"
 #include "mx-enum-types.h"
-
-#define LONG_PRESS_TIMEOUT 500
+#include "mx-settings.h"
 
 /*
  * Forward declaration for sake of MxWidgetChild
@@ -71,6 +71,9 @@ struct _MxWidgetPrivate
   MxMenu       *menu;
 
   guint         long_press_source;
+
+  guint         tooltip_timeout;
+  guint         tooltip_delay;
 };
 
 /**
@@ -95,9 +98,15 @@ enum
   PROP_TOOLTIP_TEXT,
   PROP_MENU,
 
-  PROP_DISABLED
+  PROP_DISABLED,
+
+  PROP_TOOLTIP_DELAY,
+
+  LAST_PROP
 };
 
+static GParamSpec *widget_properties[LAST_PROP];
+
 enum
 {
   LONG_PRESS,
@@ -108,11 +117,18 @@ enum
 static guint widget_signals[LAST_SIGNAL] = { 0, };
 
 static void mx_stylable_iface_init (MxStylableIface *iface);
+static ClutterScriptableIface *parent_scriptable_iface = NULL;
+static void scriptable_iface_init (ClutterScriptableIface *iface);
 
+/* Length of time in milliseconds that the cursor must be held steady
+   over a widget before the tooltip is displayed */
+#define MX_WIDGET_TOOLTIP_TIMEOUT 500
 
 G_DEFINE_ABSTRACT_TYPE_WITH_CODE (MxWidget, mx_widget, CLUTTER_TYPE_ACTOR,
                                   G_IMPLEMENT_INTERFACE (MX_TYPE_STYLABLE,
-                                                         mx_stylable_iface_init));
+                                                         mx_stylable_iface_init)
+                                  G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                         scriptable_iface_init));
 
 #define MX_WIDGET_GET_PRIVATE(obj)    (G_TYPE_INSTANCE_GET_PRIVATE ((obj), MX_TYPE_WIDGET, MxWidgetPrivate))
 
@@ -151,6 +167,10 @@ mx_widget_set_property (GObject      *gobject,
       mx_widget_set_disabled (actor, g_value_get_boolean (value));
       break;
 
+    case PROP_TOOLTIP_DELAY:
+      mx_widget_set_tooltip_delay (actor, g_value_get_int (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
@@ -192,12 +212,55 @@ mx_widget_get_property (GObject    *gobject,
       g_value_set_boolean (value, mx_widget_get_disabled (actor));
       break;
 
+    case PROP_TOOLTIP_DELAY:
+      g_value_set_int (value, mx_widget_get_tooltip_delay (actor));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
       break;
     }
 }
 
+static gboolean
+mx_widget_tooltip_timeout_cb (gpointer data)
+{
+  MxWidget *self = MX_WIDGET (data);
+  MxWidgetPrivate *priv = self->priv;
+
+  mx_widget_show_tooltip (self);
+
+  priv->tooltip_timeout = 0;
+
+  return FALSE;
+}
+
+static void
+mx_widget_remove_tooltip_timeout (MxWidget *widget)
+{
+  MxWidgetPrivate *priv = widget->priv;
+
+  if (priv->tooltip_timeout)
+    {
+      g_source_remove (priv->tooltip_timeout);
+      priv->tooltip_timeout = 0;
+    }
+}
+
+static void
+mx_widget_set_tooltip_timeout (MxWidget *widget)
+{
+  MxWidgetPrivate *priv = widget->priv;
+
+  /* Remove any existing tooltip timeout so that we can start again */
+  mx_widget_remove_tooltip_timeout (widget);
+
+  priv->tooltip_timeout =
+    clutter_threads_add_timeout (mx_widget_get_tooltip_delay (widget),
+                                 mx_widget_tooltip_timeout_cb,
+                                 widget);
+}
+
 static void
 mx_widget_dispose (GObject *gobject)
 {
@@ -250,6 +313,8 @@ mx_widget_finalize (GObject *gobject)
 {
   MxWidgetPrivate *priv = MX_WIDGET (gobject)->priv;
 
+  mx_widget_remove_tooltip_timeout (MX_WIDGET (gobject));
+
   g_free (priv->style_class);
   g_free (priv->pseudo_class);
 
@@ -271,8 +336,7 @@ mx_widget_allocate (ClutterActor          *actor,
 {
   MxWidgetPrivate *priv = MX_WIDGET (actor)->priv;
   ClutterActorClass *klass;
-  ClutterGeometry area;
-  ClutterVertex in_v, out_v;
+  ClutterActorBox frame_box = { 0, 0, box->x2 - box->x1, box->y2 - box->y1 };
 
   klass = CLUTTER_ACTOR_CLASS (mx_widget_parent_class);
   klass->allocate (actor, box, flags);
@@ -280,44 +344,47 @@ mx_widget_allocate (ClutterActor          *actor,
   /* update tooltip position */
   if (priv->tooltip)
     {
-      in_v.x = in_v.y = in_v.z = 0;
-      clutter_actor_apply_transform_to_point (actor, &in_v, &out_v);
-      area.x = out_v.x;
-      area.y = out_v.y;
+      ClutterVertex verts[4];
+      ClutterGeometry area;
+      gfloat x, y, x2, y2;
+      gint i;
 
-      in_v.x = box->x2 - box->x1;
-      in_v.y = box->y2 - box->y1;
-      clutter_actor_apply_transform_to_point (actor, &in_v, &out_v);
-      area.width = out_v.x - area.x;
-      area.height = out_v.y - area.y;
+      clutter_actor_get_abs_allocation_vertices (actor, verts);
+
+      x = y = G_MAXFLOAT;
+      x2 = y2 = -G_MAXFLOAT;
+      for (i = 0; i < G_N_ELEMENTS (verts); i++)
+        {
+          if (verts[i].x < x)
+            x = verts[i].x;
+          if (verts[i].x > x2)
+            x2 = verts[i].x;
+          if (verts[i].y < y)
+            y = verts[i].y;
+          if (verts[i].y > y2)
+            y2 = verts[i].y;
+        }
+
+      area.x = x;
+      area.y = y;
+      area.width = x2 - x;
+      area.height = y2 - y;
 
       mx_tooltip_set_tip_area (priv->tooltip, &area);
     }
 
 
 
+  /* allocate border images */
   if (priv->border_image)
-    {
-      ClutterActorBox frame_box = {
-        0,
-        0,
-        box->x2 - box->x1,
-        box->y2 - box->y1
-      };
+    clutter_actor_allocate (priv->border_image, &frame_box, flags);
 
-      clutter_actor_allocate (CLUTTER_ACTOR (priv->border_image),
-                              &frame_box,
-                              flags);
+  if (priv->old_border_image)
+    clutter_actor_allocate (priv->old_border_image, &frame_box, flags);
 
-      if (priv->old_border_image)
-        clutter_actor_allocate (priv->old_border_image, &frame_box, flags);
-    }
 
   if (priv->background_image)
     {
-      ClutterActorBox frame_box = {
-        0, 0, box->x2 - box->x1, box->y2 - box->y1
-      };
       gfloat w, h;
 
       clutter_actor_get_preferred_size (CLUTTER_ACTOR (priv->background_image),
@@ -728,16 +795,10 @@ mx_widget_enter (ClutterActor         *actor,
   MxWidget *widget = MX_WIDGET (actor);
   MxWidgetPrivate *priv = widget->priv;
 
-  if (event->source != actor)
-    return FALSE;
+  /* This is also expected to handle enter events from child actors
+     because they will bubble up */
 
-  if (priv->tooltip)
-    mx_widget_show_tooltip (widget);
-
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
-      return TRUE;
-
-  mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
+  mx_stylable_style_pseudo_class_add (MX_STYLABLE (widget), "hover");
   priv->is_hovered = TRUE;
 
   return FALSE;
@@ -750,21 +811,42 @@ mx_widget_leave (ClutterActor         *actor,
   MxWidget *widget = MX_WIDGET (actor);
   MxWidgetPrivate *priv = widget->priv;
 
-  if (event->source != actor)
+  /* If the leave event is simply moving to a child actor then we'll
+     ignore it so that the actor will retain its hover state until the
+     pointer moves to an unrelated actor. For this to work the actual
+     leave event from the child needs to bubble up to this actor */
+  if (clutter_actor_contains (actor, event->related))
     return FALSE;
 
-  if (priv->tooltip)
-    mx_tooltip_hide (priv->tooltip);
+  mx_widget_hide_tooltip (widget);
 
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
-      return TRUE;
-
-  mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "");
+  mx_widget_long_press_cancel (widget);
+  mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "active");
+  mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "hover");
   priv->is_hovered = FALSE;
 
   return FALSE;
 }
 
+static gboolean
+mx_widget_motion (ClutterActor       *actor,
+                  ClutterMotionEvent *event)
+{
+  MxWidget *widget = MX_WIDGET (actor);
+  MxWidgetPrivate *priv = widget->priv;
+
+  if (priv->tooltip && !CLUTTER_ACTOR_IS_VISIBLE (priv->tooltip))
+    {
+      /* If tooltips are in browse mode then display the tooltip immediately */
+      if (mx_tooltip_is_in_browse_mode ())
+        mx_widget_show_tooltip (widget);
+      else
+        mx_widget_set_tooltip_timeout (widget);
+    }
+
+  return FALSE;
+}
+
 static gboolean
 mx_widget_emit_long_press (MxWidget *widget)
 {
@@ -790,12 +872,16 @@ mx_widget_long_press_query (MxWidget           *widget,
 {
   MxWidgetPrivate *priv = widget->priv;
   gboolean query_result = FALSE;
+  MxSettings *settings = mx_settings_get_default ();
+  guint timeout;
+
+  g_object_get (settings, "long-press-timeout", &timeout, NULL);
 
   g_signal_emit (widget, widget_signals[LONG_PRESS], 0, event->x,
                  event->y, MX_LONG_PRESS_QUERY, &query_result);
 
   if (query_result)
-    priv->long_press_source = g_timeout_add (LONG_PRESS_TIMEOUT,
+    priv->long_press_source = g_timeout_add (timeout,
                                              (GSourceFunc) mx_widget_emit_long_press,
                                              widget);
 }
@@ -823,6 +909,31 @@ mx_widget_long_press_cancel (MxWidget *widget)
     }
 }
 
+/**
+ * mx_widget_apply_style:
+ * @widget: A #MxWidget
+ * @style: A #MxStyle
+ *
+ * Used to implement how a new style instance should be applied in the widget.
+ * For instance, setting style instance on stylable internal children.
+ *
+ * Since: 1.2
+ */
+void
+mx_widget_apply_style (MxWidget *widget,
+                       MxStyle  *style)
+{
+  MxWidgetClass *klass;
+
+  g_return_if_fail (MX_IS_WIDGET (widget));
+  g_return_if_fail (style != NULL);
+
+  klass = MX_WIDGET_GET_CLASS (widget);
+
+  if (klass->apply_style != NULL)
+    klass->apply_style (widget, style);
+}
+
 static gboolean
 mx_widget_button_press (ClutterActor       *actor,
                         ClutterButtonEvent *event)
@@ -833,11 +944,11 @@ mx_widget_button_press (ClutterActor       *actor,
       return TRUE;
 
   if (event->button == 1)
-    mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "active");
+    mx_stylable_style_pseudo_class_add (MX_STYLABLE (widget), "active");
 
   mx_widget_long_press_query (widget, event);
 
-  return TRUE;
+  return FALSE;
 }
 
 static gboolean
@@ -850,16 +961,11 @@ mx_widget_button_release (ClutterActor       *actor,
       return TRUE;
 
   if (event->button == 1)
-    {
-      if (widget->priv->is_hovered)
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "hover");
-      else
-        mx_stylable_set_style_pseudo_class (MX_STYLABLE (widget), "");
-    }
+    mx_stylable_style_pseudo_class_remove (MX_STYLABLE (widget), "active");
 
   mx_widget_long_press_cancel (widget);
 
-  return TRUE;
+  return FALSE;
 }
 
 static void
@@ -868,8 +974,7 @@ mx_widget_hide (ClutterActor *actor)
   MxWidget *widget = (MxWidget *) actor;
 
   /* hide the tooltip, if there is one */
-  if (widget->priv->tooltip)
-    mx_tooltip_hide (widget->priv->tooltip);
+  mx_widget_hide_tooltip (widget);
 
   CLUTTER_ACTOR_CLASS (mx_widget_parent_class)->hide (actor);
 }
@@ -893,6 +998,18 @@ _mx_widget_propagate_disabled (ClutterContainer *container,
 
           child_priv->parent_disabled = disabled;
 
+          /* emit the "notify" signal for the "disabled" property */
+          g_object_notify_by_pspec (G_OBJECT (container),
+                                    widget_properties[PROP_DISABLED]);
+
+
+          if (disabled)
+            mx_stylable_style_pseudo_class_add (MX_STYLABLE (child),
+                                                "disabled");
+          else
+            mx_stylable_style_pseudo_class_remove (MX_STYLABLE (child),
+                                                   "disabled");
+
           /* If this child has already been disabled explicitly,
            * we don't need to recurse through its children to set
            * the 'parent_disabled' flag, as it'll already be set.
@@ -947,7 +1064,6 @@ mx_widget_class_init (MxWidgetClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
-  GParamSpec *pspec;
 
   g_type_class_add_private (klass, sizeof (MxWidgetPrivate));
 
@@ -964,6 +1080,7 @@ mx_widget_class_init (MxWidgetClass *klass)
 
   actor_class->enter_event = mx_widget_enter;
   actor_class->leave_event = mx_widget_leave;
+  actor_class->motion_event = mx_widget_motion;
   actor_class->button_press_event = mx_widget_button_press;
   actor_class->button_release_event = mx_widget_button_release;
 
@@ -974,45 +1091,66 @@ mx_widget_class_init (MxWidgetClass *klass)
 
   /* stylable interface properties */
   g_object_class_override_property (gobject_class, PROP_STYLE, "style");
+  widget_properties[PROP_STYLE] = g_object_class_find_property (gobject_class,
+                                                                "style");
   g_object_class_override_property (gobject_class, PROP_STYLE_CLASS,
                                     "style-class");
+  widget_properties[PROP_STYLE_CLASS] =
+    g_object_class_find_property (gobject_class, "style-class");
+
   g_object_class_override_property (gobject_class, PROP_STYLE_PSEUDO_CLASS,
                                     "style-pseudo-class");
+  widget_properties[PROP_STYLE_PSEUDO_CLASS] =
+    g_object_class_find_property (gobject_class, "style-pseudo-class");
 
   /**
    * MxWidget:tooltip-text:
    *
    * text displayed on the tooltip
    */
-  pspec = g_param_spec_string ("tooltip-text",
-                               "Tooltip Text",
-                               "Text displayed on the tooltip",
-                               "",
-                               MX_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_TOOLTIP_TEXT, pspec);
+  widget_properties[PROP_TOOLTIP_TEXT] =
+    g_param_spec_string ("tooltip-text",
+                         "Tooltip Text",
+                         "Text displayed on the tooltip",
+                         "",
+                         MX_PARAM_READWRITE | MX_PARAM_TRANSLATEABLE);
+  g_object_class_install_property (gobject_class, PROP_TOOLTIP_TEXT,
+                                   widget_properties[PROP_TOOLTIP_TEXT]);
 
   /**
    * MxWidget:menu:
    *
    * #MxMenu associated with the widget.
-   * See mx_widget_show_menu().
    */
-  pspec = g_param_spec_object ("menu",
-                               "Menu",
-                               "The MxMenu associated with the widget",
-                               MX_TYPE_MENU,
-                               MX_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_MENU, pspec);
-
-  pspec = g_param_spec_boolean ("disabled",
-                               "Disabled",
-                               "Whether disabled styling should be applied and"
-                               " the widget made unreactive.",
-                               FALSE,
-                               MX_PARAM_READWRITE);
-  g_object_class_install_property (gobject_class, PROP_DISABLED, pspec);
+  widget_properties[PROP_MENU] =
+    g_param_spec_object ("menu",
+                         "Menu",
+                         "The MxMenu associated with the widget",
+                         MX_TYPE_MENU,
+                         MX_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_MENU,
+                                   widget_properties[PROP_MENU]);
 
+  widget_properties[PROP_DISABLED] = g_param_spec_boolean ("disabled",
+                                                           "Disabled",
+                                                           "Whether disabled"
+                                                           " styling should be"
+                                                           " applied and the"
+                                                           " widget made"
+                                                           " unreactive.",
+                                                           FALSE,
+                                                           MX_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_DISABLED,
+                                   widget_properties[PROP_DISABLED]);
 
+  widget_properties[PROP_TOOLTIP_DELAY] =
+    g_param_spec_int ("tooltip-delay",
+                      "Tooltip delay",
+                      "Delay time before showing the tooltip",
+                      0, G_MAXINT, MX_WIDGET_TOOLTIP_TIMEOUT,
+                      MX_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+  g_object_class_install_property (gobject_class, PROP_TOOLTIP_DELAY,
+                                   widget_properties[PROP_TOOLTIP_DELAY]);
 
   /**
    * MxWidget::long-press:
@@ -1051,6 +1189,8 @@ mx_widget_set_style (MxStylable *stylable,
     g_object_unref (priv->style);
 
   priv->style = g_object_ref_sink (style);
+
+  mx_widget_apply_style (MX_WIDGET (stylable), style);
 }
 
 static void
@@ -1068,7 +1208,8 @@ _mx_widget_set_style_class (MxStylable  *actor,
       g_free (priv->style_class);
       priv->style_class = g_strdup (style_class);
 
-      g_object_notify (G_OBJECT (actor), "style-class");
+      g_object_notify_by_pspec (G_OBJECT (actor),
+                                widget_properties[PROP_STYLE_CLASS]);
     }
 }
 
@@ -1087,17 +1228,15 @@ _mx_stylable_set_style_pseudo_class (MxStylable  *actor,
       g_free (priv->pseudo_class);
       priv->pseudo_class = g_strdup (pseudo_class);
 
-      g_object_notify (G_OBJECT (actor), "style-pseudo-class");
+      g_object_notify_by_pspec (G_OBJECT (actor),
+                                widget_properties[PROP_STYLE_PSEUDO_CLASS]);
     }
 }
 
 static const gchar*
 _mx_stylable_get_style_pseudo_class (MxStylable *actor)
 {
-  if (mx_widget_get_disabled (MX_WIDGET (actor)))
-    return "disabled";
-  else
-    return ((MxWidget *) actor)->priv->pseudo_class;
+  return ((MxWidget *) actor)->priv->pseudo_class;
 }
 
 
@@ -1218,9 +1357,7 @@ mx_stylable_iface_init (MxStylableIface *iface)
 static void
 mx_widget_init (MxWidget *actor)
 {
-  MxWidgetPrivate *priv;
-
-  actor->priv = priv = MX_WIDGET_GET_PRIVATE (actor);
+  actor->priv = MX_WIDGET_GET_PRIVATE (actor);
 
   /* set the default style */
   mx_stylable_set_style (MX_STYLABLE (actor), mx_style_get_default ());
@@ -1313,6 +1450,8 @@ mx_widget_set_has_tooltip (MxWidget *widget,
           clutter_actor_unparent (CLUTTER_ACTOR (priv->tooltip));
           priv->tooltip = NULL;
         }
+
+      mx_widget_remove_tooltip_timeout (widget);
     }
 }
 
@@ -1356,7 +1495,8 @@ mx_widget_set_tooltip_text (MxWidget    *widget,
   if (priv->tooltip)
     mx_tooltip_set_text (priv->tooltip, text);
 
-  g_object_notify (G_OBJECT (widget), "tooltip-text");
+  g_object_notify_by_pspec (G_OBJECT (widget),
+                            widget_properties[PROP_TOOLTIP_TEXT]);
 }
 
 /**
@@ -1391,22 +1531,40 @@ mx_widget_get_tooltip_text (MxWidget *widget)
 void
 mx_widget_show_tooltip (MxWidget *widget)
 {
-  gfloat x, y, width, height;
+  gint i;
+  gfloat x, y, x2, y2;
   ClutterGeometry area;
+  ClutterVertex verts[4];
 
   g_return_if_fail (MX_IS_WIDGET (widget));
 
+  /* Remove any timeout so we don't show the tooltip again */
+  mx_widget_remove_tooltip_timeout (widget);
+
   /* XXX not necceary, but first allocate transform is wrong */
 
-  clutter_actor_get_transformed_position ((ClutterActor*) widget,
-                                          &x, &y);
+  /* Work out the bounding box */
+  clutter_actor_get_abs_allocation_vertices ((ClutterActor*) widget,
+                                             verts);
 
-  clutter_actor_get_size ((ClutterActor*) widget, &width, &height);
+  x = y = G_MAXFLOAT;
+  x2 = y2 = -G_MAXFLOAT;
+  for (i = 0; i < G_N_ELEMENTS (verts); i++)
+    {
+      if (verts[i].x < x)
+        x = verts[i].x;
+      if (verts[i].x > x2)
+        x2 = verts[i].x;
+      if (verts[i].y < y)
+        y = verts[i].y;
+      if (verts[i].y > y2)
+        y2 = verts[i].y;
+    }
 
   area.x = x;
   area.y = y;
-  area.width = width;
-  area.height = height;
+  area.width = x2 - x;
+  area.height = y2 - y;
 
 
   if (widget->priv->tooltip)
@@ -1428,6 +1586,8 @@ mx_widget_hide_tooltip (MxWidget *widget)
 {
   g_return_if_fail (MX_IS_WIDGET (widget));
 
+  mx_widget_remove_tooltip_timeout (widget);
+
   if (widget->priv->tooltip)
     mx_tooltip_hide (widget->priv->tooltip);
 }
@@ -1519,7 +1679,7 @@ mx_widget_set_menu (MxWidget *widget,
  *
  * Get the object in the #MxWidget:menu property.
  *
- * Returns: The current object in the "menu" property.
+ * Returns: (transfer none): The current object in the "menu" property.
  */
 MxMenu *
 mx_widget_get_menu (MxWidget *widget)
@@ -1560,7 +1720,8 @@ mx_widget_set_disabled (MxWidget *widget,
 
       mx_stylable_style_changed (MX_STYLABLE (widget), 0);
 
-      g_object_notify (G_OBJECT (widget), "disabled");
+      g_object_notify_by_pspec (G_OBJECT (widget),
+                                widget_properties[PROP_DISABLED]);
     }
 }
 
@@ -1576,3 +1737,79 @@ mx_widget_get_disabled (MxWidget *widget)
   g_return_val_if_fail (MX_IS_WIDGET (widget), FALSE);
   return widget->priv->is_disabled || widget->priv->parent_disabled;
 }
+
+/**
+ * mx_widget_set_tooltip_delay:
+ * @widget: an #MxWidget
+ *
+ * Set the value, in milliseconds, of the "tooltip-delay" property.
+ * This is initially set to MX_WIDGET_TOOLTIP_TIMEOUT.
+ */
+void
+mx_widget_set_tooltip_delay (MxWidget *widget,
+                             guint delay)
+{
+  g_return_if_fail (MX_IS_WIDGET (widget));
+
+  if (widget->priv->tooltip_delay != delay)
+    {
+      widget->priv->tooltip_delay = delay;
+      g_object_notify_by_pspec (G_OBJECT (widget),
+                                widget_properties[PROP_TOOLTIP_DELAY]);
+    }
+}
+
+/**
+ * mx_widget_get_tooltip_delay:
+ * @widget: an #MxWidget
+ *
+ * Get the value of the "tooltip-delay" property.
+ *
+ * Returns: the current delay value in milliseconds
+ */
+guint
+mx_widget_get_tooltip_delay (MxWidget *widget)
+{
+  g_return_if_fail (MX_IS_WIDGET (widget));
+
+  return widget->priv->tooltip_delay;
+}
+
+/* Support translateable strings from JSON */
+static void
+widget_scriptable_set_custom_property (ClutterScriptable *scriptable,
+                                       ClutterScript     *script,
+                                       const gchar       *name,
+                                       const GValue      *value)
+{
+  GParamSpec *pspec;
+
+  pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (scriptable), name);
+
+  if (pspec && pspec->flags & MX_PARAM_TRANSLATEABLE &&
+      pspec->value_type == G_TYPE_STRING)
+    {
+      g_object_set (scriptable, name,
+                    gettext (g_value_get_string (value)), NULL);
+    }
+  else
+    {
+      /* chain up */
+      if (parent_scriptable_iface->set_custom_property)
+        parent_scriptable_iface->set_custom_property (scriptable, script,
+                                                      name,
+                                                      value);
+    }
+}
+
+static void
+scriptable_iface_init (ClutterScriptableIface *iface)
+{
+  parent_scriptable_iface = g_type_interface_peek_parent (iface);
+
+  if (!parent_scriptable_iface)
+    parent_scriptable_iface = g_type_default_interface_peek
+                                          (CLUTTER_TYPE_SCRIPTABLE);
+
+  iface->set_custom_property = widget_scriptable_set_custom_property;
+}
